/// <reference types="react" />
export declare type FoundationFormValue = unknown;
export interface FoundationFormValueArray extends Array<FoundationFormValueObject> {
}
export interface FoundationFormValueObject extends Record<string, FoundationFormValues> {
}
export declare type FoundationFormValues = FoundationFormValue | FoundationFormValueArray | FoundationFormValueObject;
export interface FoundationFormFieldErrors extends Array<[string, string]> {
}
export interface FoundationFormErrors extends Record<string, FoundationFormFieldErrors> {
}
export interface FoundationFormValidationContext {
    key: string;
    value: FoundationFormValue;
    values: FoundationFormValueObject;
}
export interface FoundationFormValidator {
    (context: FoundationFormValidationContext): FoundationFormFieldErrors;
}
export interface FoundationFormValidatorArray<T> extends Array<FoundationFormValidatorObject<T>> {
}
export declare type FoundationFormValidatorObject<T> = {
    [P in keyof T]?: T[P] extends FoundationFormValueArray ? FoundationFormValidatorArray<T[P][number]> : T[P] extends FoundationFormValueObject ? FoundationFormValidatorObject<T[P]> : FoundationFormValidator;
};
export declare type FoundationFormValidators<T> = FoundationFormValidatorObject<T> | FoundationFormValidatorArray<T> | FoundationFormValidator;
export interface FoundationFormChangeEventExtras {
    name: string;
}
export interface FoundationFormChangeEventHandler {
    (value: unknown): void;
}
export interface FoundationFormFieldActions<T> {
    change: (value: T) => Promise<void>;
    reset: () => Promise<void>;
}
export interface FoundationFormFieldsActions<T> {
    change: (value: T) => Promise<void>;
    reset: () => Promise<void>;
}
export interface FoundationFormFieldArrayActions {
    add: (initialValue: FoundationFormValueObject) => Promise<void>;
    remove: (index: number) => Promise<void>;
    removeAll: () => Promise<void>;
    reset: () => Promise<void>;
}
export interface FoundationFormField<T> {
    $$actions: FoundationFormFieldActions<T>;
    errors: FoundationFormFieldErrors;
    hasChanges: boolean;
    hasErrors: boolean;
    initialValue: T | undefined;
    name: string;
    onChange: FoundationFormChangeEventHandler;
    submitValue: T;
    value: T;
}
export interface FoundationFormFieldArray<T> extends Array<FoundationFormFields<T>> {
    $$actions: FoundationFormFieldArrayActions;
}
export declare type FoundationFormFieldsProps<T> = {
    [P in keyof T]: T[P] extends FoundationFormValueArray ? FoundationFormFieldArray<T[P][number]> : T[P] extends FoundationFormValueObject ? FoundationFormFields<T[P]> : FoundationFormField<T[P]>;
};
export declare type FoundationFormFields<T> = {
    $$actions: FoundationFormFieldsActions<T>;
} & FoundationFormFieldsProps<T>;
export interface FoundationFormContextType<T> {
    errors: FoundationFormErrors;
    fields: FoundationFormFields<T>;
    hasErrors: boolean;
    isSubmitting: boolean;
    reset: () => void;
    submit: () => void;
    values: T;
}
export interface FoundationFormProviderProps<T> {
    children: React.ReactNode;
    context: FoundationFormContextType<T>;
}
export interface FoundationFormConsumerProps<T> {
    children: (context: FoundationFormContextType<T>) => React.ReactElement | null;
}
export interface FoundationFormChildren<TFormValues extends FoundationFormValueObject> {
    (context: FoundationFormContextType<TFormValues>): React.ReactElement;
}
export interface FoundationFormResetEventHandler {
    (): void;
}
export interface FoundationFormSubmitHandler<TFormValues extends FoundationFormValueObject, TData = unknown> {
    (values: TFormValues): Promise<TData>;
}
export interface FoundationFormSubmitErrorEventHandler<TFormValues extends FoundationFormValueObject> {
    (errors: FoundationFormErrors, values: TFormValues, isClientSideValidationError: boolean): void;
}
export interface FoundationFormSubmitSuccessEventHandler<TFormValues extends FoundationFormValueObject, TData = unknown> {
    (data: TData, values: TFormValues): void;
}
export interface FoundationFormProps<TFormValues extends FoundationFormValueObject, TData = unknown> {
    children: FoundationFormChildren<TFormValues>;
    enableReinitialize?: boolean;
    initialValues: TFormValues;
    keepSuccessfulIsSubmitting?: boolean;
    mergeValuesOnReinitialize?: boolean;
    onReset: FoundationFormResetEventHandler;
    onSubmitError: FoundationFormSubmitErrorEventHandler<TFormValues>;
    onSubmitSuccess: FoundationFormSubmitSuccessEventHandler<TFormValues, TData>;
    resetOnSubmitSuccess?: boolean;
    submit: FoundationFormSubmitHandler<TFormValues, TData>;
    validators?: FoundationFormValidatorObject<TFormValues>;
    wrapInForm?: boolean;
}
export interface FoundationFormFieldsState {
    addArrayElement: (name: string, initialValue: FoundationFormValueObject) => Promise<void>;
    change: (name: string, value: unknown) => Promise<void>;
    removeArrayElementAt: (name: string, index: number) => Promise<void>;
    removeAllArrayElements: (name: string) => Promise<void>;
}
export interface FoundationFormValuesState<T> {
    initialValues: T;
    rawValues: T;
    submitValues: T;
}
