import type {
    FoundationFormValuesState,
} from '../types';

import change from './change';


test('can change primitive', () => {
    const setState = change('field1', 'new value');

    const prevState = {
        initialValues: {
            field1: 'value',
        },
        rawValues: {
            field1: 'value',
        },
        submitValues: {
            field1: 'value',
        },
    };

    const nextState = setState(prevState);

    expect(nextState).toEqual({
        rawValues: {
            field1: 'new value',
        },
        submitValues: {
            field1: 'new value',
        },
    });
});

test('can change array', () => {
    const setState = change('field1', [{ field11: 'new value' }]);

    const prevState = {
        initialValues: {
            field1: [{ field11: 'value' }],
        },
        rawValues: {
            field1: [{ field11: 'value' }],
        },
        submitValues: {
            field1: [{ field11: 'value' }],
        },
    };

    const nextState = setState(prevState);

    expect(nextState).toEqual({
        rawValues: {
            field1: [{ field11: 'new value' }],
        },
        submitValues: {
            field1: [{ field11: 'new value' }],
        },
    });
});

test('can change object', () => {
    const setState = change('field1', { field11: 'new value' });

    const prevState = {
        initialValues: {
            field1: { field11: 'value' },
        },
        rawValues: {
            field1: { field11: 'value' },
        },
        submitValues: {
            field1: { field11: 'value' },
        },
    };

    const nextState = setState(prevState);

    expect(nextState).toEqual({
        rawValues: {
            field1: { field11: 'new value' },
        },
        submitValues: {
            field1: { field11: 'new value' },
        },
    });
});

test('can change money object', () => {
    const newValue = {
        rawValue: {
            currency: 'SEK',
            value: '1',
        },
        value: {
            currency: 'SEK',
            value: 1,
        },
    };

    const setState = change('field1', newValue);

    const prevState: FoundationFormValuesState<{
        field1: {
            currency: string;
            value: number | string;
        };
    }> = {
        initialValues: {
            field1: {
                currency: 'SEK',
                value: 0,
            },
        },
        rawValues: {
            field1: {
                currency: 'SEK',
                value: '0',
            },
        },
        submitValues: {
            field1: {
                currency: 'SEK',
                value: 0,
            },
        },
    };

    const nextState = setState(prevState);

    expect(nextState).toEqual({
        rawValues: {
            field1: {
                currency: 'SEK',
                value: '1',
            },
        },
        submitValues: {
            field1: {
                currency: 'SEK',
                value: 1,
            },
        },
    });
});

test('can change object with rawValue and value (string)', () => {
    const newValue = {
        rawValue: 'new value',
        value: 'new value',
    };

    const setState = change('field1', newValue);

    const prevState = {
        initialValues: {
            field1: 'value',
        },
        rawValues: {
            field1: 'value',
        },
        submitValues: {
            field1: 'value',
        },
    };

    const nextState = setState(prevState);

    expect(nextState).toEqual({
        rawValues: {
            field1: 'new value',
        },
        submitValues: {
            field1: 'new value',
        },
    });
});

test('can change object with rawValue and value (number)', () => {
    const newValue = {
        rawValue: '1',
        value: 1,
    };

    const setState = change('field1', newValue);

    const prevState: FoundationFormValuesState<{
        field1: number | string;
    }> = {
        initialValues: {
            field1: 0,
        },
        rawValues: {
            field1: '1',
        },
        submitValues: {
            field1: 0,
        },
    };

    const nextState = setState(prevState);

    expect(nextState).toEqual({
        rawValues: {
            field1: '1',
        },
        submitValues: {
            field1: 1,
        },
    });
});
