import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import money from '@billogram/money';
import getErrorKey from './getErrorKey';

function isObject(value) {
  return value instanceof Object && !money.is(value);
}

function isArrayFormField(value) {
  return Array.isArray(value);
}

function isArrayFormFieldOrUndefined(value) {
  return Array.isArray(value) || value === void 0;
}

function isObjectFormField(value) {
  return !Array.isArray(value) && isObject(value);
}

function isObjectFormFieldOrUndefined(value) {
  return !Array.isArray(value) && (isObject(value) || value === void 0);
}

function toArray(initialValues, submitValues, errors, prefix, state) {
  return function map(value, index) {
    var initialValue = initialValues !== void 0 ? initialValues[index] : void 0;
    var submitValue = submitValues[index];
    var name = "".concat(prefix, ".").concat(index); // NOTE: it is impossible to satisfy this rules due to the recursive nature of this algorithm.
    // eslint-disable-next-line @typescript-eslint/no-use-before-define

    return traverse(initialValue, submitValue, value, errors, name, state);
  };
}

function traverse(initialValues, submitValues, rawValues, errors, prefix, state) {
  function reducer(fields, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    var initialValue = initialValues !== void 0 ? initialValues[key] : void 0;
    var submitValue = submitValues[key];
    var name = prefix !== '' ? "".concat(prefix, ".").concat(key) : key;

    if (isArrayFormFieldOrUndefined(initialValue) && isArrayFormField(submitValue) && isArrayFormField(value)) {
      var array = value.map(toArray(initialValue, submitValue, errors, name, state));
      var _actions = {
        add: function add(initialValue) {
          return state.addArrayElement(name, initialValue);
        },
        remove: function remove(index) {
          return state.removeArrayElementAt(name, index);
        },
        removeAll: function removeAll() {
          return state.removeAllArrayElements(name);
        },
        reset: function reset() {
          return initialValue !== void 0 ? state.change(name, initialValue) : Promise.resolve();
        }
      };
      Object.defineProperty(array, '$$actions', {
        value: _actions,
        writable: false
      });
      return _objectSpread(_objectSpread({}, fields), {}, _defineProperty({}, key, array));
    }

    if (isObjectFormFieldOrUndefined(initialValue) && isObjectFormField(submitValue) && isObjectFormField(value)) {
      var obj = traverse(initialValue, submitValue, value, errors, name, state);
      return _objectSpread(_objectSpread({}, fields), {}, _defineProperty({}, key, obj));
    }

    var errorKey = getErrorKey(name, value); // I don't feel comfortable changing this at the moment.
    // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition

    var formFieldErrors = errors[errorKey] !== void 0 ? errors[errorKey] : [];
    var actions = {
      change: function change(value) {
        return state.change(name, value);
      },
      reset: function reset() {
        return state.change(name, initialValue);
      }
    };
    return _objectSpread(_objectSpread({}, fields), {}, _defineProperty({}, key, {
      $$actions: actions,
      errors: formFieldErrors,
      hasChanges: initialValue !== value,
      hasErrors: formFieldErrors.length > 0,
      initialValue: initialValue,
      name: name,
      onChange: function onChange(value) {
        state.change(name, value);
      },
      submitValue: submitValue,
      value: value
    }));
  }

  var formFields = Object.entries(rawValues).reduce(reducer, {});
  var actions = {
    change: function change(value) {
      return state.change(prefix, value);
    },
    reset: function reset() {
      return initialValues !== void 0 ? state.change(prefix, initialValues) : Promise.resolve();
    }
  };
  return Object.assign(formFields, {
    $$actions: actions
  });
}

export default function getFormFields(initialValues, submitValues, rawValues, errors, state) {
  return traverse(initialValues, submitValues, rawValues, errors, '', state);
}
//# sourceMappingURL=getFormFields.js.map