import type {
    FoundationFormFieldErrors,
} from '../types';

import mergeShallow from './mergeShallow';


function always(): FoundationFormFieldErrors { return []; }

test('nothing', () => {
    const validators = {};
    const values = {};

    const result = mergeShallow(validators, values);
    expect(result).toHaveLength(0);
});

test('no validators', () => {
    const validators = {};
    const values = { key: 'value' };

    const result = mergeShallow(validators, values);
    expect(result).toHaveLength(1);
    expect(result[0]).toEqual(['key', void 0, 'value']);
});

test('validators', () => {
    const validators = { key: always };
    const values = { key: 'value' };

    const result = mergeShallow(validators, values);

    expect(result).toHaveLength(1);
    expect(result[0]).toEqual(['key', always, 'value']);
});

test('some validators', () => {
    const validators = { key1: always };
    const values = {
        key1: 'value1',
        key2: 'value2',
    };

    const result = mergeShallow(validators, values);

    expect(result).toHaveLength(2);
    expect(result[0]).toEqual(['key1', always, 'value1']);
    expect(result[1]).toEqual(['key2', void 0, 'value2']);
});
