import all from './all';
import always from './always';
import never from './never';
import validate from './validate';


describe('always', () => {
    test('primitive', () => {
        const values = {
            field1: 'value',
        };

        const validators = {
            field1: always(),
        };

        expect(() => { validate(values, validators); }).not.toThrow();
    });

    test('object', () => {
        const values = {
            field1: {
                field11: 'value',
            },
        };

        const validators = {
            field1: {
                field11: always(),
            },
        };

        expect(() => { validate(values, validators); }).not.toThrow();
    });

    test('array', () => {
        const values = {
            field1: [{
                field11: 'value',
            }],
        };

        const validators = {
            field1: [{
                field11: always(),
            }],
        };

        expect(() => { validate(values, validators); }).not.toThrow();
    });
});

describe('never', () => {
    test('primitive', () => {
        const values = {
            field1: 'value',
        };

        const validators = {
            field1: never({ message: 'never' }),
        };

        expect(() => { validate(values, validators); }).toThrow();
    });

    test('object', () => {
        const values = {
            field1: {
                field11: 'value',
            },
        };

        const validators = {
            field1: {
                field11: never({ message: 'never' }),
            },
        };

        expect(() => { validate(values, validators); }).toThrow();
    });

    test('array', () => {
        const values = {
            field1: [{
                field11: 'value',
            }],
        };

        const validators = {
            field1: [{
                field11: never({ message: 'never' }),
            }],
        };

        expect(() => { validate(values, validators); }).toThrow();
    });
});

test('multiple validators', () => {
    const values = {
        field1: 'value',
    };

    const validators = {
        field1: all(
            never({ message: 'never' }),
            never({ message: 'never' }),
        ),
    };

    expect(() => { validate(values, validators); }).toThrow();
});
